package client

import (
	"git.gz.internal.jumaiyx.cn/jm/jmproto/conf"
	"github.com/go-kratos/kratos/v2/config"
	"github.com/go-kratos/kratos/v2/log"
	"github.com/pkg/errors"
	clientv3 "go.etcd.io/etcd/client/v3"
	"strings"
	"sync"
	"time"
)

var (
	HostNullError = errors.New("host empty is not allowed")
	DialFailed    = errors.New("dial fail")
)
var (
	clinet *clientv3.Client
	mux    sync.Mutex
)

type Data struct {
	Etcd conf.Etcd
}

type Etcd struct {
	Host     string
	Username string
	Password string
	Timeout  int32
	Pool     struct {
		Min int32
		Max int32
	}
}

// Client Link etcd
func Client(c clientv3.Config) (*clientv3.Client, error) {
	log.Infof("etcd host:%v username: %s password:%s timeout:%d", c.Endpoints, c.Username, c.Password, c.DialTimeout)
	client, err := clientv3.New(c)
	if err != nil {
		log.Error(DialFailed)
		return nil, err
	}
	return client, nil
}

func Conf(bc *conf.Etcd) (*clientv3.Config, error) {
	if bc.Host == "" {
		log.Warn("host empty is not allowed")
		return nil, HostNullError
	}
	cli := &clientv3.Config{
		Endpoints: strings.Split(bc.Host, ","),
	}
	if bc.Username != "" {
		cli.Username = bc.Username
	}
	if bc.Password != "" {
		cli.Password = bc.Password
	}
	cli.DialTimeout = 3 * time.Second
	if bc.Timeout != 0 {
		cli.DialTimeout = time.Duration(bc.Timeout) * time.Second
	}
	return cli, nil
}

func NewConfClient(c config.Config) (*clientv3.Client, error) {
	var bc Data
	if err := c.Scan(&bc); err != nil {
		return nil, err
	}
	return getClient(&bc.Etcd)
}

func NewClient(bc *conf.Etcd) (*clientv3.Client, error) {
	return getClient(bc)
}

func getClient(bc *conf.Etcd) (*clientv3.Client, error) {
	mux.Lock()
	defer mux.Unlock()
	if clinet == nil {
		cli, err := Conf(bc)
		if err != nil {
			return nil, err
		}
		clinet, err = Client(*cli)
		if err != nil {
			return nil, err
		}
	}
	return clinet, nil
}
