package discver

import (
	"git.gz.internal.jumaiyx.cn/jm/jmproto/conf"
	"git.gz.internal.jumaiyx.cn/pkg/go-etcd/discver/pool"
	"github.com/go-kratos/kratos/v2/log"
	"time"
)

type RegistryConfig struct {
	MaxOpen     int
	MinOpen     int
	MaxLifetime time.Duration
	Conf        *conf.Etcd
}

func NewDiscver(c *RegistryConfig) *RegistryConfig {
	if c.MaxOpen == 0 {
		c.MaxOpen = 10
	}
	if c.MinOpen == 0 {
		c.MinOpen = 5
	}
	if c.MaxLifetime == 0 {
		c.MaxLifetime = 5 * time.Second
	}
	return c
}

func (dis *RegistryConfig) Pool() (*pool.GenericPool, error) {
	genericPool, err := pool.NewGenericPool(dis.MinOpen, dis.MaxOpen, dis.MaxLifetime, func() (pool.Poolable, error) {
		confRegistry, err := NewRegistry(dis.Conf)
		if err != nil {
			log.Errorf(pool.ErrCreatePoolFailed, err)
			return nil, err
		}
		return confRegistry, nil
	})
	if err != nil {
		log.Errorf(pool.ErrCreatePoolFailed, err)
	}
	return genericPool, err
}
